navIList = Components.interfaces.navIList;
/******************************************************************************
 * navURL component
 ******************************************************************************/
function navURL() 
{
  var pbs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
  this._branch = pbs.getBranch("urlnav.");
  this._global = Components.classes["@mozilla.org/browser/global-history;1"].getService(Components.interfaces.nsIGlobalHistory);
};

navURL.prototype = {
  _url: "",
  _history: false,
  _valid: false,
  _branch: null,
  _global: null,
  
  get url() { return this._url; },
  get history() { return this._history; },
  get valid() { return this._valid; },
  
  set url(aValue) {
    this._url = aValue;
    
    //set history
    if (this._branch.getBoolPref("history.enabled"))
      this._history = this._global.isVisited(this._url);
    else
      this._history = false;
                
    //set valid
    if (this._branch.getBoolPref("valid.enabled"))
      this._valid = this._validate(this._url);
    else
      this._valid = false;    
  },
  
  _validate: function(aURL)
  {
    var ios = Components.classes["@mozilla.org/network/io-service;1"].createInstance(Components.interfaces.nsIIOService);
    var uri = ios.newURI(aURL, null, null);
    if (!uri.schemeIs("http") && !uri.schemeIs("https"))
      return false;
      
    //request head for url
    var channel = ios.newChannel(aURL, null, null);   
    channel = channel.QueryInterface(Components.interfaces.nsIHttpChannel);
    channel.requestMethod = "HEAD";
    channel.open();
    
    //return true if response was okay
    try {
      if (channel.responseStatus == 200)
        return true;
      else
        return false;
    } catch(e) {
      return false;
    };
  },
     
  ///////////////////////////
  // nsIClassInfo  
  getInterfaces: function(aCount)
  {
    var ifaces = new Array();
    ifaces.push(Components.interfaces.navIURL);
    ifaces.push(Components.interfaces.nsISupports);
    ifaces.push(Components.interfaces.nsIClassInfo);
    aCount.value = ifaces.length;
    return ifaces;
  },
  
  getHelperForLanguage: function(aLanguage) { return null; },
  get contractID() { return "@mozdev.org/urlnav/url;1"; },
  get classID() { return Components.ID("{24D13C3B-9904-4d1f-AD3D-521182620A6A}"); },
  get classDescription() { return "Location navigator url"; },
  get implementationLanguage() { return Components.interfaces.nsIProgrammingLanguage.JAVASCRIPT; },
  get flags() { return Components.interfaces.nsIClassInfo.THREADSAFE; },
   
  ///////////////////////////
  // nsISupports  
  QueryInterface: function(aIID)
  {
    if (!aIID.equals(Components.interfaces.navIURL) &&
        !aIID.equals(Components.interfaces.nsIClassInfo) &&
        !aIID.equals(Components.interfaces.nsISupports))
       throw Components.results.NS_ERROR_NO_INTERFACE;
    return this;
  }
};
/******************************************************************************
 * navList component
 ******************************************************************************/
function navList() 
{
  this._start = Components.classes["@mozdev.org/urlnav/url;1"].createInstance(Components.interfaces.navIURL);
  this._current = Components.classes["@mozdev.org/urlnav/url;1"].createInstance(Components.interfaces.navIURL);
  this._up = Components.classes["@mozdev.org/urlnav/url;1"].createInstance(Components.interfaces.navIURL);
  this._down = Components.classes["@mozdev.org/urlnav/url;1"].createInstance(Components.interfaces.navIURL);     
};

navList.prototype = {
  _start: null,
  _current: null,
  _up: null,
  _down: null,
  _builder: null,
  _startIndex: 0,
  _endIndex: 0,
  _value: "",
  _referrer: null,
  _branch: null,
  
  get start() { return this._start; },
  get current() { return this._current; },
  get up() { return this._up; },
  get down() { return this._down; },
  get builder() { return this._builder; },
  get startIndex() { return this._startIndex; },
  get endIndex() { return this._endIndex; },
  get referrer() { return this._referrer; },
    
  init: function(aURL, aBuilder, aStartIndex, aEndIndex, aReferrer) 
  {
    pbs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService); 
    this._branch = pbs.getBranch("urlnav.");
    this._start.url = aURL;
    this._current.url = aURL;
    this._builder = aBuilder;
    this._startIndex = aStartIndex;
    this._endIndex = aEndIndex;
    this._referrer = aReferrer;
    this._value = aURL.substring(aStartIndex, aEndIndex);
    
    //build up and down
    this._up.url = this._buildURL(navIList.MOVE_UP);
    this._down.url = this._buildURL(navIList.MOVE_DOWN);
  },
  
  move: function(aEvent, aDirection, aWindow)
  {
    //determine where to open link
    var where = this._where(aEvent);

    //increment value and determine history
    var history = false;    
    switch (aDirection) {
      case navIList.MOVE_START:
        history = this._branch.getBoolPref("history.start");
        this._value = this.start.url.substring(this._startIndex, this._endIndex);
        break;    
      case navIList.MOVE_UP:
        history = this._branch.getBoolPref("history.updown");
        this._value = this._builder.increment(this._value, aDirection);
        break;
      case navIList.MOVE_DOWN:
        history = this._branch.getBoolPref("history.updown");
        this._value = this._builder.increment(this._value, aDirection);
        break;
    };
    
    //set urls
    this._current.url = this._buildURL(null);
    this._up.url = this._buildURL(navIList.MOVE_UP);
    this._down.url = this._buildURL(navIList.MOVE_DOWN);
    
    //open the current url
    this._open(this._current.url, where, history, aWindow);    
  },
  
  _where: function(aEvent)
  {
    if (!aEvent) {
      aEvent = new Object();
      aEvent.shiftKey = false;
      aEvent.ctrlKey = false;
      aEvent.button = 0;
    };
    
    var shift = aEvent.shiftKey;
    var ctrl =  aEvent.ctrlKey;
    var middle = (aEvent.button == 1);
    
    if (ctrl || middle) {
      if (shift)
        return "tabshifted";
      else
        return "tab";
    } else if (shift)
      return "window";
    else
      return "current"; 
  },
   
  _open: function(aURL, aWhere, aHistory, aWindow)
  {
    //get the internal browser 
    var browser = aWindow.document.getElementById("content");
    
    //open the link
    switch (aWhere) {
      case "window":
        aWindow.openDialog("chrome://browser/content/browser.xul", "_blank", "chrome,all,dialog=no", aURL, null, null);
        break;
      case "tab":
      case "tabshifted":
        var tab = browser.addTab(aURL);
        if (aWhere == "tab") {
          browser.selectedTab = tab;
          win.content.focus();
        };        
        break;
      case "current":
      default:
        if (aHistory) {
          var sessionHistory = browser.webNavigation.sessionHistory;    
          for (var i=sessionHistory.count -1; i >= 0; --i) {
            var entry = sessionHistory.getEntryAtIndex(i, false);
            if (entry.URI.spec == aURL) {
              browser.webNavigation.gotoIndex(i);
              return;
            };    
          };
          browser.webNavigation.loadURI(aURL, Components.interfaces.nsIWebNavigation.LOAD_FLAGS_NONE, this._referrer, null, null);
        } else {
          browser.webNavigation.loadURI(aURL, Components.interfaces.nsIWebNavigation.LOAD_FLAGS_NONE, this._referrer, null, null);
        };
        break;
    };    
  },
  
  _buildURL: function(aDirection)
  {
    //split start url into parts
    var prefix = this._start.url.substring(0, this._startIndex);
    var suffix = this._start.url.substring(this._endIndex, this._start.url.length);
    
    //set the value
    var value;
    if (aDirection)
      value = this._builder.increment(this._value, aDirection);
    else
      value = this._value;
      
    //combine the url back together
    return prefix + value + suffix;
  },
    
  ///////////////////////////
  // nsIClassInfo  
  getInterfaces: function(aCount)
  {
    var ifaces = new Array();
    ifaces.push(Components.interfaces.navIList);
    ifaces.push(Components.interfaces.nsISupports);
    ifaces.push(Components.interfaces.nsIClassInfo);
    aCount.value = ifaces.length;
    return ifaces;
  },
  
  getHelperForLanguage: function(aLanguage) { return null; },
  get contractID() { return "@mozdev.org/urlnav/list;1"; },
  get classID() { return Components.ID("{D416356F-A69C-4585-8ABE-7F572EFEB06B}"); },
  get classDescription() { return "Location navigator list"; },
  get implementationLanguage() { return Components.interfaces.nsIProgrammingLanguage.JAVASCRIPT; },
  get flags() { return Components.interfaces.nsIClassInfo.THREADSAFE; },
   
  ///////////////////////////
  // nsISupports  
  QueryInterface: function(aIID)
  {
    if (!aIID.equals(Components.interfaces.navIList) &&
        !aIID.equals(Components.interfaces.nsIClassInfo) &&
        !aIID.equals(Components.interfaces.nsISupports))
       throw Components.results.NS_ERROR_NO_INTERFACE;
    return this;
  }
};
/******************************************************************************
 * navBuilderNum component
 ******************************************************************************/
function navBuilderNum() 
{
  var pbs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
  this._branch = pbs.getBranch(this._prefix);
  var sbs = Components.classes["@mozilla.org/intl/stringbundle;1"].getService(Components.interfaces.nsIStringBundleService);    
  this._bundle = sbs.createBundle("chrome://urlnav/locale/urlnav.properties");
  
  //setup the name of the builder
  this._name = this._bundle.GetStringFromName(this._prefix + "name");
  
  //setup the builders properties
  this._properties = Components.classes["@mozilla.org/hash-property-bag;1"].createInstance(Components.interfaces.nsIWritablePropertyBag);
  this._properties.setProperty("min", this._branch.getIntPref("min"));
  this._properties.setProperty("max", this._branch.getIntPref("max"));
  this._properties.setProperty("step", this._branch.getIntPref("step"));
  this._properties.setProperty("trim", this._branch.getBoolPref("trim"));       
};

navBuilderNum.prototype = {
  _name: "",
  _pane: "chrome://urlnav/content/options/numeric.xul",
  _overlay: "chrome://urlnav/content/overlays/numeric.xul",
  _prefix: "urlnav.builder.num.",
  _properties: null,
  _bundle: null,
  _branch: null,
  
  get name() { return this._name; },
  get pane() { return this._pane; },
  get overlay() { return this._overlay; },  
  get properties() { return this._properties; },
    
  increment: function(aValue, aDirection)
  {
    if (isNaN(Number(aValue)))
      throw Components.Exception(this._bundle.GetStringFromName(this._prefix + "error"), Components.results.NS_ERROR_ABORT); 
        
    var length = aValue.length;
    var value = Number(aValue);
    var min = this._properties.getProperty("min");
    var max = this._properties.getProperty("max")
    
    //determine decimal positions and leading zeros     
    var precision = 0;
    var decimals = 0;
    if (aValue.indexOf(".") != -1) {
      decimals = aValue.length - aValue.indexOf(".") -1;
      precision = aValue.indexOf(".");
    };
      
    //increment value  
    var step = this._properties.getProperty("step");
    switch (aDirection) {
      case navIList.MOVE_UP:
      default:
        value += step;
        if (value > max)
          value = min;
        break;
      case navIList.MOVE_DOWN:
        value -= step;
        if (value < min)
          value = max;
        break;
    };
    
    //set correct number of decimal places
    value = value.toFixed(decimals);
    
    //trim of leading zeros if not there to begin with
    var strValue = String(value);      
    if (precision == 0 && decimals > 0 && strValue.substring(0,1) == "0") 
      strValue = strValue.substring(strValue.indexOf("."), strValue.length);
      
    //return value if not padding zeros  
    if (this._properties.getProperty("trim"))
      return strValue;
      
    //pad with zeros
    while (strValue.length < length)
      strValue = "0" + strValue;
      
    //return full value
    return strValue;      
  },
  
  guess: function(aTextBox)
  {
    //set full selection
    aTextBox.select();
    
    //get length and value
    var value = aTextBox.getAttribute("value");
    var endIndex = value.length-1;
    
    //nothing in textbox
    if (endIndex < 0)
      return;
      
    //loop thorugh text looking for numeric portion
    do {
      //non numeric value
      if (isNaN(parseInt(value.charAt(endIndex))))
        continue
        
      //numeric value  
      var startIndex = endIndex-1;
      var found = true;
      do {
        if (isNaN(value.charAt(startIndex)))
          break;
      } while (--startIndex);
      break;
    } while (--endIndex);
      
    //set selection range
    if (found) {
      ++startIndex;
      aTextBox.setSelectionRange(startIndex, endIndex + 1);
    };  
  },
        
  ///////////////////////////
  // nsIClassInfo  
  getInterfaces: function(aCount)
  {
    var ifaces = new Array();
    ifaces.push(Components.interfaces.navIBuilder);
    ifaces.push(Components.interfaces.nsISupports);
    ifaces.push(Components.interfaces.nsIClassInfo);
    aCount.value = ifaces.length;
    return ifaces;
  },
  
  getHelperForLanguage: function(aLanguage) { return null; },
  get contractID() { return "@mozdev.org/urlnav/builder;1?type=num"; },
  get classID() { return Components.ID("{35F0A87E-908D-4349-B8E7-813CC0520F61}"); },
  get classDescription() { return "Location navigator numeric builder"; },
  get implementationLanguage() { return Components.interfaces.nsIProgrammingLanguage.JAVASCRIPT; },
  get flags() { return Components.interfaces.nsIClassInfo.THREADSAFE; },
   
  ///////////////////////////
  // nsISupports  
  QueryInterface: function(aIID)
  {
    if (!aIID.equals(Components.interfaces.navIBuilder) &&
        !aIID.equals(Components.interfaces.nsIClassInfo) &&
        !aIID.equals(Components.interfaces.nsISupports))
       throw Components.results.NS_ERROR_NO_INTERFACE;
    return this;
  }
};
/******************************************************************************
 * navBuilderAlpha component
 ******************************************************************************/
function navBuilderAlpha() 
{
  var pbs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
  this._branch = pbs.getBranch(this._prefix);
  var sbs = Components.classes["@mozilla.org/intl/stringbundle;1"].getService(Components.interfaces.nsIStringBundleService);    
  this._bundle = sbs.createBundle("chrome://urlnav/locale/urlnav.properties");
  
  //setup the name of the builder
  this._name = this._bundle.GetStringFromName(this._prefix + "name");
  
  //setup the builders properties
  this._properties = Components.classes["@mozilla.org/hash-property-bag;1"].createInstance(Components.interfaces.nsIWritablePropertyBag);
  this._properties.setProperty("min", this._branch.getCharPref("min"));
  this._properties.setProperty("max", this._branch.getCharPref("max"));
  this._properties.setProperty("step", this._branch.getIntPref("step"));
};

navBuilderAlpha.prototype = {
  _name: "",
  _pane: "chrome://urlnav/content/options/alpha.xul",
  _overlay: "chrome://urlnav/content/overlays/alpha.xul",
  _prefix: "urlnav.builder.alpha.",
  _properties: null,
  _bundle: null,
  _branch: null,

  get name() { return this._name; },  
  get pane() { return this._pane; },
  get overlay() { return this._overlay; },
  get properties() { return this._properties; },
    
  increment: function(aValue, aDirection)
  {
    //do sanity test
    if (aValue.length > 1)
      throw Components.Exception(this._bundle.GetStringFromName(this._prefix + "error"), Components.results.NS_ERROR_ABORT);
       
    //get character code for alpha values
    var value = aValue.charCodeAt(0);
    var upper = (value >= 65 && value <= 90);
    value = aValue.toLowerCase().charCodeAt(0);

    //make sure we are in the alpha realm    
    if (value < 97 || value > 122)
      throw Components.Exception(this._bundle.GetStringFromName(this._prefix + "char"), Components.results.NS_ERROR_ABORT);
          
    var min = this._properties.getProperty("min").toLowerCase().charCodeAt(0);    
    var max = this._properties.getProperty("max").toLowerCase().charCodeAt(0);
    
    //increment character code
    var step = this._properties.getProperty("step");
    switch (aDirection) {
      case navIList.MOVE_UP:
      default:
        value += step;
        if (value > max)
          value = min;
        break;
      case navIList.MOVE_DOWN:
        value -= step;
        if (value < min)
          value = max;       
        break;
    };

    //return string of character code
    var strValue = String.fromCharCode(value);
    return (upper) ? strValue.toUpperCase() : strValue;   
  },
  
  guess: function(aTextBox)
  {
    //set full selection
    aTextBox.select();
    
    //get length and value
    var value = aTextBox.getAttribute("value");
    var endIndex = value.length-1;
    
    //nothing in textbox
    if (endIndex < 0)
      return;
      
    //loop thorugh text looking for alpha portion
    do {
      //numeric value
      if (!isNaN(parseInt(value.charAt(endIndex))))
        continue
        
      //non numeric value  
      var startIndex = endIndex-1;
      var found = true;
      break;
    } while (--endIndex);
      
    //set selection range
    if (found) {
      ++startIndex;
      aTextBox.setSelectionRange(startIndex, endIndex + 1);
    };  
  },
          
  ///////////////////////////
  // nsIClassInfo  
  getInterfaces: function(aCount)
  {
    var ifaces = new Array();
    ifaces.push(Components.interfaces.navIBuilder);
    ifaces.push(Components.interfaces.nsISupports);
    ifaces.push(Components.interfaces.nsIClassInfo);
    aCount.value = ifaces.length;
    return ifaces;
  },
  
  getHelperForLanguage: function(aLanguage) { return null; },
  get contractID() { return "@mozdev.org/urlnav/builder;1?type=alpha"; },
  get classID() { return Components.ID("{E6A9F44F-CD2D-4fe7-9D4C-5E7CB5E83B7C}"); },
  get classDescription() { return "Location navigator alpha builder"; },
  get implementationLanguage() { return Components.interfaces.nsIProgrammingLanguage.JAVASCRIPT; },
  get flags() { return Components.interfaces.nsIClassInfo.THREADSAFE; },
   
  ///////////////////////////
  // nsISupports  
  QueryInterface: function(aIID)
  {
    if (!aIID.equals(Components.interfaces.navIBuilder) &&
        !aIID.equals(Components.interfaces.nsIClassInfo) &&
        !aIID.equals(Components.interfaces.nsISupports))
       throw Components.results.NS_ERROR_NO_INTERFACE;
    return this;
  }
};
/******************************************************************************
 * XPCOM Functions for construction and registration
 ******************************************************************************/
var gModule = {
  _firstTime: true,
  registerSelf: function(aCompMgr, aFileSpec, aLocation, aType)
  {
    if (this._firstTime) {
      this._firstTime = false;
      throw Components.results.NS_ERROR_FACTORY_REGISTER_AGAIN;
    };
    
    aCompMgr = aCompMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
    var catMgr = Components.classes["@mozilla.org/categorymanager;1"].getService(Components.interfaces.nsICategoryManager);
    for (var key in this._objects) {
      var obj = this._objects[key];    
      aCompMgr.registerFactoryLocation(obj.classID, obj.className, obj.contractID, aFileSpec, aLocation, aType);
      if (obj.builder)
        catMgr.addCategoryEntry("urlnav-builders", obj.className, obj.contractID, true, true);    
    };
  },

  unregisterSelf: function(aCompMgr, aLocation, aType)
  {
    aCompMgr = aCompMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
    var catMgr = Components.classes["@mozilla.org/categorymanager;1"].getService(Components.interfaces.nsICategoryManager);
    for (var key in this._objects) {
      var obj = this._objects[key];    
      aCompMgr.unregisterFactoryLocation(obj.classID, aLocation);
      if (obj.builder)
        catMgr.deleteCategoryEntry("urlnav-builders", obj.className, true);           
    };      
  },
  
  getClassObject: function(aCompMgr, aCID, aIID)
  {
    if (!aIID.equals(Components.interfaces.nsIFactory))
      throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

    for (var key in this._objects) {
      var obj = this._objects[key];    
      if (aCID.equals(obj.classID))
        return obj.factory;
    };

    throw Components.results.NS_ERROR_NO_INTERFACE;
  },

  _objects: {
    url: {
      classID: navURL.prototype.classID,
      contractID: navURL.prototype.contractID,
      className: navURL.prototype.classDescription,
      builder: false,      
      factory: {
        createInstance: function (aOuter, aIID) {
          if (aOuter != null)
            throw Components.results.NS_ERROR_NO_AGGREGATION;
          return (new navURL()).QueryInterface(aIID);
        }       
      }
    },
    
    list: {
      classID: navList.prototype.classID,
      contractID: navList.prototype.contractID,
      className: navList.prototype.classDescription,
      builder: false,      
      factory: {
        createInstance: function (aOuter, aIID) {
          if (aOuter != null)
            throw Components.results.NS_ERROR_NO_AGGREGATION;
          return (new navList()).QueryInterface(aIID);
        }       
      }    
    },
    
    num: {
      classID: navBuilderNum.prototype.classID,
      contractID: navBuilderNum.prototype.contractID,
      className: navBuilderNum.prototype.classDescription,
      builder: true,      
      factory: {
        createInstance: function (aOuter, aIID) {
          if (aOuter != null)
            throw Components.results.NS_ERROR_NO_AGGREGATION;
          return (new navBuilderNum()).QueryInterface(aIID);
        }       
      }
    },
    
    alpha: {
      classID: navBuilderAlpha.prototype.classID,
      contractID: navBuilderAlpha.prototype.contractID,
      className: navBuilderAlpha.prototype.classDescription,
      builder: true,      
      factory: {
        createInstance: function (aOuter, aIID) {
          if (aOuter != null)
            throw Components.results.NS_ERROR_NO_AGGREGATION;
          return (new navBuilderAlpha()).QueryInterface(aIID);
        }       
      }
    }
  },
  
  canUnload: function(aCompMgr) { return true; }
};

function NSGetModule(aCompMgr, aFileSpec) { return gModule; }